@@ -23,6 +23,7 @@ uses the plugins, not the other way around.
 """
 
 from gettext import gettext as _
+import glib
 import gnomeapplet
 import gobject
 import gtk
@@ -901,7 +902,10 @@ class NotifyPopup (gobject.GObject):
         import pynotify
 
         if self.__notify is not None:
-            self.__notify.close ()
+            try:
+                self.__notify.close ()
+            except glib.GError:
+                pass        # might happen if notification was already destroyed
 
         if self.__plugin is not None and self.__plugin.title is not None and self.__conf.client.get_bool (self.__key):
             # title is automatically escaped, but body is not
@@ -955,7 +959,10 @@ class NotifyPopup (gobject.GObject):
 
     def __notification_config_changed_cb (self, client, id, entry, unused):
         if entry.value is not None and not entry.value.get_bool () and self.__notify is not None:
-            self.__notify.close ()
+            try:
+                self.__notify.close ()
+            except glib.GError:
+                pass        # might happen if notification was already destroyed
             self.__notify = None
 
 
