@@ -194,7 +194,7 @@ class Conf:
         Ultimately, the schema determines which will be used.
         """
 
-        escaped = plugin.player_name.replace (" ", "-")
+        escaped = plugin.internal_name.replace (" ", "-")
         legacy_key = self.resolve_key ("%s_%s" % (escaped.lower (), key))
         if self.client.get (legacy_key) is not None:
             return legacy_key
