@@ -64,6 +64,9 @@ struct _MaScrollerPrivate
 	MaScrollerField next;
 	gint offset;
 
+	gint scaled_width;
+	gint scaled_height;
+
 	guint source;
 };
 
@@ -113,6 +116,9 @@ static void ma_scroller_direction_changed (GtkWidget *widget,
 static void ma_scroller_size_request (GtkWidget *widget,
 				      GtkRequisition *requisition);
 
+static void ma_scroller_size_allocate (GtkWidget *widget,
+				       GtkAllocation *allocation);
+
 static gboolean ma_scroller_expose_event (GtkWidget *widget,
 					  GdkEventExpose *event);
 
@@ -190,6 +196,7 @@ ma_scroller_class_init (MaScrollerClass *klass)
 	widget_class->style_set = ma_scroller_style_set;
 	widget_class->direction_changed = ma_scroller_direction_changed;
 	widget_class->size_request = ma_scroller_size_request;
+	widget_class->size_allocate = ma_scroller_size_allocate;
 	widget_class->expose_event = ma_scroller_expose_event;
 
 	g_object_class_install_property (object_class,
@@ -228,6 +235,9 @@ ma_scroller_init (MaScroller *scroller, MaScrollerClass *klass)
 	priv->next = FIELD_TITLE;
 	priv->offset = 0;
 
+	priv->scaled_width = -1;
+	priv->scaled_height = -1;
+
 	priv->source = 0;
 }
 
@@ -478,6 +488,8 @@ ma_scroller_create_layout (MaScroller *scroller, const gchar *text)
 
 	layout = pango_layout_new (priv->context);
 	pango_layout_set_ellipsize (layout, PANGO_ELLIPSIZE_END);
+	pango_layout_set_width (layout, priv->scaled_width);
+	pango_layout_set_height (layout, priv->scaled_height);
 	pango_layout_set_text (layout, text, -1);
 
 	return layout;
@@ -728,6 +740,41 @@ ma_scroller_size_request (GtkWidget *widget, GtkRequisition *requisition)
 	}
 }
 
+static void
+ma_scroller_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
+{
+	MaScrollerPrivate *priv = GET_PRIVATE (widget);
+	guint i;
+
+	if (GTK_WIDGET_CLASS (parent_class)->size_allocate != NULL)
+		GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+
+	/*
+	 * pango_layout_set_(width|height) doesn't consider the context's
+	 * matrix, so swap width and height if vertical.
+	 */
+
+	if (VERTICAL (priv->orientation))
+	{
+		priv->scaled_width = allocation->height * PANGO_SCALE;
+		priv->scaled_height = allocation->width * PANGO_SCALE;
+	}
+	else
+	{
+		priv->scaled_width = allocation->width * PANGO_SCALE;
+		priv->scaled_height = allocation->height * PANGO_SCALE;
+	}
+
+	for (i = 0; i < NUM_FIELDS; ++i)
+	{
+		if (priv->layouts[i] != NULL)
+		{
+			pango_layout_set_width (priv->layouts[i], priv->scaled_width);
+			pango_layout_set_height (priv->layouts[i], priv->scaled_height);
+		}
+	}
+}
+
 static gboolean
 ma_scroller_expose_event (GtkWidget *widget, GdkEventExpose *event)
 {
