@@ -31,7 +31,7 @@ templates to see what these are.
 from gettext import gettext as _
 import gobject
 import gtk
-from gtk import glade
+from gtk import gdk, glade
 
 import os.path
 
@@ -291,6 +291,29 @@ class PluginBase (gobject.GObject):
         self.art = None
 
 
+    def _load_art (self, source):
+        """
+        Load an image from a file or a file:// URI, and set it as self.art;
+        set self.art to None if failure.
+
+        It also handles the cases where the "source" is None or an empty
+        string.
+        """
+
+        try:
+            if source is not None:
+                if source.startswith ("file://"):
+                    source = source[7:]
+
+                if source.startswith ("/"):
+                    self.art = gdk.pixbuf_new_from_file (source)
+                else:
+                    self.art = None
+        except gobject.GError:
+            # File couldn't be loaded.
+            self.art = None
+
+
 class DBusPlugin (PluginBase):
     """
     Base class for all plugins that rely on D-Bus to communicate with the
@@ -506,10 +529,10 @@ class MPRISPlugin (DBusPlugin):
 
 
     def toggle_playback (self):
-        if self.__status == self.STATUS_STOPPED:
-            self.__player.Play ()
-        else:
+        if self.__status == self.STATUS_PLAYING:
             self.__player.Pause ()
+        else:
+            self.__player.Play ()
 
 
     def previous (self):
@@ -521,13 +544,14 @@ class MPRISPlugin (DBusPlugin):
 
 
     def __get_metadata (self, metadata):
-        print "DEBUG: MPRIS %s: __get_metadata:" % self.__dbus_name
-        for key in metadata:
-            print "DEBUG:    ", key, "=>", metadata[key]
-
-        self.title = metadata.get ("title", None) or None
+        # VLC hides stream metadata under "nowplaying"
+        if metadata.has_key ("nowplaying"):
+            self.title = metadata.get ("nowplaying") or None
+            self.album = metadata.get ("album", metadata.get ("title", None)) or None
+        else:
+            self.title = metadata.get ("title", None) or None
+            self.album = metadata.get ("album", None) or None
         self.artist = metadata.get ("artist", None) or None
-        self.album = metadata.get ("album", None) or None
 
         if metadata.has_key ("time"):
             self.duration = metadata.get ("time")
@@ -541,43 +565,47 @@ class MPRISPlugin (DBusPlugin):
         self.rating = metadata.get ("rating", -1.0)
 
         if metadata.has_key ("arturl"):
-            path = metadata["arturl"]
-            if path.startswith ("file://"):
-                path = path[7:]
-            self.art = gdk.pixbuf_new_from_file (path)
+            self._load_art (metadata["arturl"])
         else:
             self.art = None
 
 
     def __get_status (self, status):
-        print "DEBUG: MPRIS %s: __get_status:" % self.__dbus_name, status
-
         # XXX: The spec (http://wiki.xmms2.xmms.se/index.php/MPRIS) claims
         #      this is a list of four ints, but some players only bother
         #      with the first one
 
         import dbus
         if type (status) == dbus.Struct:
-            self.__status = status[0]
+            new_status = status[0]
         else:
-            self.__status = status
-        self.playing = (self.__status == self.STATUS_PLAYING)
-        if self.__status == self.STATUS_STOPPED:
-            self._set_no_song ()
+            new_status = status
+        if self.__status != new_status:
+            self.__status = new_status
+            self.playing = (self.__status == self.STATUS_PLAYING)
+            if self.__status == self.STATUS_STOPPED:
+                self._set_no_song ()
 
 
     def __get_caps (self, caps):
-        print "DEBUG: MPRIS %s: __get_caps:" % self.__dbus_name, caps
-        self.__caps = caps
-        if not (caps & self.CAN_PROVIDE_METADATA):
-            self._set_no_song ()
+        if self.__caps != caps:
+            self.__caps = caps
+            if not (caps & self.CAN_PROVIDE_METADATA):
+                self._set_no_song ()
 
 
     def __position_get (self, elapsed):
-        print "DEBUG: MPRIS %s: __position_get:" % self.__dbus_name, elapsed
         if elapsed > 0 or self.__status != self.STATUS_STOPPED:
             self.elapsed = elapsed / 1000
 
+            # If playing a radio stream, periodically check for updated
+            # metadata, since we can't count on their being a signal sent
+            # when this happens.
+
+            if self.duration <= 0 and self.elapsed % 10 == 0:
+                self.__player.GetMetadata (reply_handler=self.__get_metadata,
+                                           error_handler=self._report_dbus_error)
+
 
     def __poll (self):
         self.__player.PositionGet (reply_handler=self.__position_get,
