@@ -19,7 +19,6 @@ import musicapplet.defs
 import musicapplet.player
 
 import gobject
-from gtk import gdk
 
 
 INFO = {
@@ -205,10 +204,7 @@ class BansheePlugin (musicapplet.player.DBusPlugin):
 
 
     def __set_art (self, art_uri):
-        if art_uri is not None and len (art_uri) > 0:
-            self.art = gdk.pixbuf_new_from_file (art_uri)
-        else:
-            self.art = None
+        self._load_art (art_uri)
 
 
     ##################################################################
