@@ -37,7 +37,15 @@ def search_model (model, column, value):
 def make_property (name):
     """
     Create a Python property that wraps a GObject property.
+
+    Note that the setter first checks whether the value is actually changing,
+    to suppress uninteresting notifications of a property change.
     """
 
-    return property (lambda self: self.get_property (name),
-                     lambda self, value: self.set_property (name, value))
+    def getter (self):
+        return self.get_property (name)
+    def setter (self, value):
+        if value != self.get_property (name):
+            self.set_property (name, value)
+
+    return property (getter, setter)
